unit ssd1305u;

interface

uses
 Math,
 ssd1305hu,
 wiringpi;

type
 TDisplayMode = (dmOriginal,dmPlus90,dmPlus180,dmPlus270);

procedure SSD1305_begin();

procedure SSD1305_clear();

procedure SSD1305_SetDisplayMode(DisplayMode : TDisplayMode);

procedure SSD1305_pixel(X     : Integer;
                        Y     : Integer;
                        Color : Byte);

procedure SSD1305_char1616(X      : Byte;
                           Y      : Byte;
                           ChChar : Byte);

procedure SSD1305_char3216(X      : Byte;
                           Y      : Byte;
                           ChChar : Byte);

procedure SSD1305_char(X     : Byte;
                       Y     : Byte;
                       Ascii : Char;
                       Size  : Byte;
                       Mode  : Byte);

procedure SSD1305_string(X    : Byte;
                         Y    : Byte;
                         Str  : String;
			 Size : Byte;
                         Mode : Byte);

procedure SSD1305_bitmap(X        : Byte;
                         Y        : Byte;
                         PBmp     : PByte;
                         ChWidth  : Byte;
                         ChHeight : Byte);

procedure SSD1305_display();

implementation

const
 CHANNEL            = 0;
 WIDTH              = 128;
 HEIGHT             = 32;
 RST                = 6;
 DC                 = 5;
 cnVisualBufferSize = 512;

var
 buffer       : array [0 .. cnVisualBufferSize - 1] of Byte;
 _displayMode : TDisplayMode = dmOriginal;

procedure command(Cmd : Byte);
begin
 digitalWrite(DC,LOW);
 wiringPiSPIDataRW(CHANNEL,@Cmd,1);
end;

procedure SSD1305_begin();
begin
 pinMode(RST,OUTPUT);
 pinMode(DC,OUTPUT);
 wiringPiSPISetup(CHANNEL,2000000);    //2M

 digitalWrite(RST,HIGH);
 delay(10);
 digitalWrite(RST,LOW);
 delay(10);
 digitalWrite(RST,HIGH);

 command($AE);  //--turn off oled panel
 command($04);  //--turn off oled panel
 command($10);  //--turn off oled panel
 command($40);  //---set low column address
 command($81);  //---set high column address
 command($80);  //--set start line address  Set Mapping RAM Display Start Line ($00~$3F)
 command($A1);  //--set contrast control register
 command($A6);  // Set SEG Output Current Brightness
 command($A8);  //--Set SEG/Column Mapping     $a0Г—ГіГ“Г’В·ВґГ–Гѓ $a1Г•ГЅВіВЈ
 command($1F);  //Set COM/Row Scan Direction   $c0Г‰ГЏГЏГ‚В·ВґГ–Гѓ $c8Г•ГЅВіВЈ
 command($C8);  //--set normal display
 command($D3);  //--set multiplex ratio(1 to 64)
 command($00);  //--1/64 duty
 command($D5);  //-set display offset	Shift Mapping RAM Counter ($00~$3F)
 command($F0);  //-not offset
 command($d8);  //--set display clock divide ratio/oscillator frequency
 command($05);  //--set divide ratio, Set Clock as 100 Frames/Sec
 command($D9);  //--set pre-charge period
 command($C2);  //Set Pre-Charge as 15 Clocks & Discharge as 1 Clock
 command($DA);  //--set com pins hardware configuration
 command($12);
 command($DB);  //--set vcomh
 command($08);  //Set VCOM Deselect Level
 command($AF);  //-Set Page Addressing Mode ($00/$01/$02)
end;

procedure SSD1305_clear();
begin
 Initialize(buffer);
end;

procedure SSD1305_SetDisplayMode(DisplayMode : TDisplayMode);
begin
 _displayMode := DisplayMode;
end;

procedure SSD1305_pixel(X     : Integer;
                        Y     : Integer;
                        Color : Byte);
 var
  bufIndex : Integer;
  realX    : Integer;
  realY    : Integer;

begin
 realX := X;
 realY := Y;

 case _displayMode of
  dmPlus90 :
   begin
    realX := WIDTH - Y;
    realY := X;
   end;

  dmPlus180 :
   begin
    realX := WIDTH - X;
    realY := HEIGHT - Y;
   end;

  dmPlus270 :
   begin
    realX := Y;
    realY := HEIGHT - X;
   end;
 end;

 if (realX >= 0) and (realX < WIDTH) and
    (realY >= 0) and (realY < HEIGHT) then
 begin
  bufIndex := realX + (realY div 8) * WIDTH;

  if ((bufIndex >= 0) and (bufIndex < cnVisualBufferSize)) then
  begin
   buffer[bufIndex] := IfThen(color = 1,
                              buffer[bufIndex] or (1 shl (realY mod 8)),
                              buffer[bufIndex] and (255 - (1 shl (realY mod 8))));
  end;
 end;
end;

procedure SSD1305_char1616(X      : Byte;
                           Y      : Byte;
                           ChChar : Byte);
 var
  i,j    : Byte;
  chTemp : Byte;
  y0     : Byte;
  b      : Byte;

begin
 y0 := Y;

 for i := 0 to 31 do
 begin
  chTemp := Font1612[chChar - $30][i];
  b      := $80;

  for j := 0 to 7 do
  begin
   SSD1305_pixel(X,Y,IfThen(Boolean(chTemp and b),1,0));

   b := b div 2;

   Inc(Y);

   if ((Y - y0) = 16) then
   begin
    Y := y0;
    Inc(X);

    Break;
   end;
  end;
 end;
end;

procedure SSD1305_char3216(X      : Byte;
                           Y      : Byte;
                           ChChar : Byte);
 var
  i,j    : Byte;
  chTemp : Byte;
  y0     : Byte;
  b      : Byte;

begin
 y0 := Y;

 for i := 0 to 63 do
 begin
  chTemp := Font3216[chChar - $30][i];
  b      := $80;

  for j := 0 to 7 do
  begin
   SSD1305_pixel(X,Y,IfThen(Boolean(chTemp and b),1,0));

   b := b div 2;

   Inc(Y);

   if ((Y - y0) = 32) then
   begin
    Y := y0;
    Inc(X);

    Break;
   end;
  end;
 end;
end;

procedure SSD1305_char(X     : Byte;
                       Y     : Byte;
                       Ascii : Char;
                       size  : Byte;
                       Mode  : Byte);
 var
  i,j,y0 : Byte;
  temp   : Byte;
  ch     : Byte;
  b      : Byte;

begin
 y0 := Y;
 ch := Ord(Ascii) - Ord(' ');

 for i := 0 to Size - 1 do
 begin
  if (Size = 12) then temp := IfThen(Mode = 1,Font1206[ch][i],255 - Font1206[ch][i])
                 else temp := IfThen(Mode = 1,Font1608[ch][i],255 - Font1608[ch][i]);
  b := $80;

  for j := 0 to 7 do
  begin
   SSD1305_pixel(X,Y,IfThen(Boolean(temp and b),1,0));

   b := b div 2;

   Inc(Y);

   if ((Y - y0) = Size) then
   begin
    Y := y0;
    Inc(X);

    break;
   end;
  end;
 end;
end;

procedure SSD1305_string(X    : Byte;
                         Y    : Byte;
                         Str  : String;
			 Size : Byte;
                         Mode : Byte);
 var
  i : Integer;

begin
 if (Str <> '') then
 begin
  for i := 1 to Length(Str) do
  begin
   if (X > (WIDTH - Size div 2)) then
   begin
    X := 0;
    Y := Y + Size;

    if (Y > (HEIGHT - Size)) then
    begin
     Y := 0;
     X := 0;
    end;
   end;

   SSD1305_char(X,Y,Str[i],Size,Mode);

   X := X + Size div 2;
  end;
 end;
end;

procedure SSD1305_bitmap(X        : Byte;
                         Y        : Byte;
                         PBmp     : PByte;
                         ChWidth  : Byte;
                         ChHeight : Byte);
 var
  i,j       : Byte;
  byteWidth : Byte;

begin
 byteWidth := (ChWidth + 7) div 8;

 for j := 0 to ChHeight - 1 do
 begin
  for i := 0 to ChWidth - 1 do
  begin
   if Boolean(PByte(PBmp + j * byteWidth + i div 8)^ and (128 shr (i and 7))) then
   begin
    SSD1305_pixel(X + i,Y + j,1);
   end;
  end;
 end;
end;

procedure SSD1305_display();
 var
  page : Byte;
  pBuf : PByte;

begin
 pBuf := @buffer[0];

 for page := 0 to 3 do
 begin
  command($B0 + page);     // set page address
  command($04);            // set low column address
  command($10);            // set high column address

  // write data

  digitalWrite(DC,HIGH);
  wiringPiSPIDataRW(CHANNEL,pBuf,WIDTH);

  Inc(pBuf,WIDTH);
 end;
end;

end.

